"""
Author: Robert Best
Description: Tkinter application that allows the user to select
a rectangular portion of their screen using the `grab()` function,
which will be returned as a coordinate tuple. If run as a script,
the coordinates will be written to a JSON config file, default file
is `out.txt` but a different name can be passed in on the command line.
"""
import sys
import json
import tkinter as tk


def _mouseMove(c, rect, x, y):
    """Moves the lower right corner of a rectangle on
    a Tkinter Canvas to the given (x, y)-coordinates,
    effectively stretching the rectangle

    Arguments:
        c: Tkinter Canvas
            Any Tkinter Canvas object
        rect: int
            The identifier of a rectangle on the canvas,
            returned by canvas.create_rectangle
        x: int
            Destination x-coordinate
        y: int
            Destination y-coordinate
    """
    x1, y1, _, _ = c.coords(rect)
    c.coords(rect, x1, y1, x, y)


def grab():
    """Opens a Tkinter window and allows the user to
    click and drag a rectangular selection, then returns
    the coordinates of the selection
    """
    bbox = [0, 0, 0, 0]

    # Set up Tkinter window
    root = tk.Tk()
    root.wait_visibility(root)
    root.attributes('-alpha', 0.1)
    root.attributes('-fullscreen', True)

    # Create Tkinter Canvas and Rect object
    c = tk.Canvas(root)
    c.pack(fill='both', expand=True)
    rect = c.create_rectangle(0, 0, 0, 0, fill="blue")

    def end():
        nonlocal bbox
        bbox = c.coords(rect)
        root.destroy()

    # Set event bindings and start Tkinter main loop
    root.bind("<ButtonPress-1>", lambda e: c.coords(rect, e.x_root, e.y_root, e.x_root, e.y_root))
    root.bind("<B1-Motion>", lambda e: _mouseMove(c, rect, e.x_root, e.y_root))
    root.bind("<ButtonRelease-1>", lambda e: end())
    root.bind("<Escape>", lambda e: end())
    root.mainloop()

    return bbox



import sys
import time
import json
import pyscreenshot as ig
import pytesseract as pt
from pynput.keyboard import Key, Listener
from pynput.mouse import Button, Controller
from Levenshtein._levenshtein import distance as levenshtein


def _keyDown(e):
    """Callback for keyboard listener, closes
    the listener if the Enter key is pressed
    """
    if e == Key.enter:
        print('Exiting')
        return False
    return True


def match(screenText, targetText="Fishing Bobber", threshold=5):
    """Takes in some text and iterates through it line by line,
    checking to see if any of the lines contain something similar
    to the target text using Levenshtein distance

    Arguments:
        screenText: string
            The text to search for a match in
        targetText: string="Fishing Bobber"
            The text to search for
        threshold: int=5
            The maximum acceptable Levenshtein distance to
            be considered a match
    
    Returns:
        True if a match was found, False otherwise
    
    Examples:
        >>> match("Fishing Bobber")
        True
        >>> match("Fishing Gobber")
        True
        >>> match("Fishing")
        False 
        >>> match("Some text Fishing Bobber some more text")
        True
    """
    ngram_size = len(targetText.split())
    for line in screenText.splitlines():
        line = line.split()
        for i in range(len(line)-ngram_size+1):
            ngram = " ".join(line[i:i+ngram_size])
            if levenshtein(ngram, targetText) <= threshold:
                return True
    return False

def start(bbox, allowed_error=5, tesspath=""):
    if tesspath != "":
        pt.pytesseract.tesseract_path = tesspath

    mouse = Controller()
    listener = Listener(on_press=_keyDown)
    listener.start()

    timeout = 45 # Fish always appears within 45 seconds, if we wait longer than this then recast the line
    cast_time = time.monotonic()
    while listener.is_alive():
        # Grab portion of screen defined in config file & send it to tesseract
        screen = ig.grab(bbox=bbox)
        screen_text = pt.image_to_string(screen)
        if match(screen_text, threshold=allowed_error) or time.monotonic() - cast_time > timeout:
            # Either fish was found or timeout was exceeded, reel in the line and cast it again
            mouse.click(Button.right, 2)
            cast_time = time.monotonic()
        time.sleep(0.25)

    listener.join()



from setuptools import setup, find_packages

with open('requirements.txt', 'r') as fp:
    reqs = [x.strip() for x in fp.readlines()]

with open('README.md', 'r') as fp:
    readme = fp.read()

setup(
    name='mc_autofisher',
    version='0.4',
    description='Automated fishing program for Minecraft',
    long_description=readme,
    classifiers=[
      'Programming Language :: Python :: 3.6'
    ],
    keywords='python python3 minecraft',
    url='https://github.com/r-best/MinecraftAutoFisher',
    author='Robert Best',
    author_email='bobbyisbest3@gmail.com',
    license='gpl-3.0',
    packages=find_packages(),
    install_requires=reqs,
    include_package_data=True,
    zip_safe=False
)



import pytest
from ..fisher import match


class TestMatch():
    def test_none(self):
        assert not match("")

    def test_exact(self):
        assert match("Fishing Bobber")
    
    def test_extra(self):
        assert match("some text Fishing Bobber some text")

    def test_oneTypo(self):
        assert match("Fishing Gobber")
    
    def test_extraMultiline(self):
        assert match("""
            this is a multiline string that contains\n
            the target Fishing Bobber text along\n
            with some other text that doesn't matter
        """)
    
    def test_maxDistanceDefault(self):
        assert match("Fishing B")
        assert match("Fishing Bmmmmm")
    
    def test_maxDistanceCustom(self):
        assert match("Fishing m", threshold=6)
        assert match("Fishing mmmmmm", threshold=6)
    
    def test_distanceTooLargeDefault(self):
        assert not match("Fishing mmmmmm")
    
    def test_distanceTooLargeCustom(self):
        assert not match("Fishing mmmmmm", threshold=4)
